using System;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace blue_dish_te
{
	// Surface
	// Z         Y
	// ^  c      ^  h
	// | b*d     | * *
	// |  a      |  l
	// +----->X  +------>X
	//
	
	public enum Surface
	{
		a = 1,
		b = 2,
		c = 3,
		d = 4,
		h = 5,
		l = 6
	}
	enum Direction
	{
		Plus = 1,
		Minus = -1,
		NA = 0
	}
	
	class SurfaceBuilder
	{
		public VertexBuffer Vertex;
		public IndexBuffer Index;
		
		// direction
		//  -
		// - + 
		//  +
		
		public SurfaceBuilder(Device device, Surface surface, int widthCount, int heightCount, float sizeX, float sizeY, float gap, Vector3 normals)
		{
			float tickX = sizeX / widthCount;
			float tickY = sizeY / heightCount;

			int IndexCount = widthCount * heightCount * 2 * 3;
			int VertexCount = (widthCount + 1) * (heightCount + 1);

			float x = 0, y = 0, z = 0;
			float maxx = 0, maxy = 0, maxz = 0;
			Direction dirx = Direction.Plus, diry = Direction.Plus, dirz = Direction.Plus;
			
			
			Vertex = new VertexBuffer(typeof(CustomVertex.PositionNormalTextured), (widthCount + 1) * (heightCount + 1), device, Usage.WriteOnly, CustomVertex.PositionNormalTextured.Format, Pool.Managed);
			CustomVertex.PositionNormalTextured[] buffer = (CustomVertex.PositionNormalTextured[]) Vertex.Lock(0, 0);
			
			switch(surface)
			{
				case Surface.a:
					y = sizeY;
					diry = Direction.Minus;
					maxx = sizeX;
					z = gap;
					dirz = Direction.NA;
					break;
				case Surface.b:
					y = sizeY;
					diry = Direction.Minus;
					z = sizeX;
					dirz = Direction.Minus;
					x = gap;
					dirx = Direction.NA;
					break;
				case Surface.c:
					x = sizeX;
					dirx = Direction.Minus;
					y = sizeY;
					diry = Direction.Minus;
					z = gap;
					dirz = Direction.NA;
					break;
				case Surface.d:
					y = sizeY;
					diry = Direction.Minus;
					maxz = sizeX;
					x = gap;
					dirz = Direction.NA;
					break;
				case Surface.h:
					x = sizeX;
					dirx = Direction.Minus;
					maxz = sizeY;
					y = gap;
					break;
				case Surface.l:
					break;
			}
			
			Vertex.Unlock();
			

			// Define indices
			
			Index = new IndexBuffer(typeof(ushort), IndexCount, device, Usage.WriteOnly, Pool.Managed);
			ushort[] indices = (ushort[]) Index.Lock(0 ,0);
			int i = 0;
			for (y = 0; y < heightCount; y++)
			{
				for (x = 0; x < widthCount; x++)
				{
					indices[i] = (ushort) (2 * y * (widthCount + 1) + 2 * x);
					i++;
					indices[i] = (ushort) ((2 * y + 1) * (widthCount + 1) + 2 * x + 1);
					i++;
					indices[i] = (ushort) ((2 * y + 1) * (widthCount + 1) + 2 * x);
					i++;

					indices[i] = (ushort) (2 * y * (widthCount + 1) + 2 * x);
					i++;
					indices[i] = (ushort) ((y + 1) * (widthCount + 1) + 2 * x + 1);
					i++;
					indices[i] = (ushort) (y * (widthCount + 1) + 2 * x);
					i++;
				}
			}
			Index.Unlock();
		}

		private CustomVertex.PositionNormalTextured[] CreateSurfaceXY(int Count, float startX, float stopX, float startY, float stopY, float z, float tickX, float tickY, Direction dirx, Direction diry, Vector3 normals)
		{
			float x = startX;
			float y = startY;
			CustomVertex.PositionNormalTextured[] v = new CustomVertex.PositionNormalTextured[Count];
			int i = 0;

			while (Math.Abs(y - stopY) > tickY / 2f)
			{
				while (Math.Abs(x - stopX) > tickX / 2f)
				{
					v[i] = new CustomVertex.PositionNormalTextured(new Vector3(x, y, z), normals, Math.Abs(startY - y), Math.Abs(startX - x));
					x = x + tickX * (float) dirx;
					i++;
				}
				x = 0;
				y = y + tickY * (float) diry;
			}
			return v;
		}
		
		private CustomVertex.PositionNormalTextured[] CreateSurfaceYZ(int Count, float startY, float stopY, float startZ, float stopZ, float x, float tickY, float tickZ, Direction diry, Direction dirz, Vector3 normals)
		{
			float y = startY;
			float z = startZ;
			CustomVertex.PositionNormalTextured[] v = new CustomVertex.PositionNormalTextured[Count];
			int i = 0;

			while (Math.Abs(y - stopY) > tickY / 2f)
			{
				while (Math.Abs(z - stopZ) > tickZ / 2f)
				{
					v[i] = new CustomVertex.PositionNormalTextured(new Vector3(x, y, z), normals, Math.Abs(startY - y), Math.Abs(startZ - z));
					z = z + tickZ * (float) dirz;
					//if (Math.Abs(z - stopZ) > tickZ / 2f)
					i++;
				}
				z = 0;
				y = y + tickY * (float) diry;
			}
			return v;
		}
		private CustomVertex.PositionNormalTextured[] CreateSurfaceZX(int Count, float startZ, float stopZ, float startX, float stopX, float y, float tickZ, float tickX, Direction dirz, Direction dirx, Vector3 normals)
		{
			float z = startZ;
			float x = startX;
			CustomVertex.PositionNormalTextured[] v = new CustomVertex.PositionNormalTextured[Count];
			int i = 0;

			while (Math.Abs(z - stopZ) > tickZ / 2f)
			{
				while (Math.Abs(x - stopX) > tickX / 2f)
				{
					v[i] = new CustomVertex.PositionNormalTextured(new Vector3(x, y, z), normals, Math.Abs(startX - x), Math.Abs(startZ - z));
					x = x + tickX * (float) dirx;
					i++;
				}
				z = z + tickZ * (float) dirz;
			}
			return v;
		}
	}
}
